@@ -11,4 +11,8 @@ from lightly.data.collate import SimCLRCollateFunction
 from lightly.data.collate import MoCoCollateFunction
 from lightly.data.collate import MultiCropCollateFunction
 from lightly.data.collate import SwaVCollateFunction
-from lightly.data.collate import imagenet_normalize
\ No newline at end of file
+from lightly.data.collate import imagenet_normalize
+from lightly.data._video import VideoError
+from lightly.data._video import EmptyVideoError
+from lightly.data._video import NonIncreasingTimestampError
+from lightly.data._video import UnseekableTimestampError
\ No newline at end of file
