@@ -1,10 +1,11 @@
 """The lightly.models package provides model implementations.
 
-*Note that the high-level building blocks will be deprecated with 
-lightly version 1.2.0. Instead, use low-level building blocks to build the
-models yourself. 
-Have a look at the benchmark code to see a reference implementation:*
-`lightly benchmarks <https://github.com/lightly-ai/lightly/tree/master/docs/source/getting_started/benchmarks>`_
+Note that the high-level building blocks will be deprecated with 
+lightly version 1.3.0. Instead, use low-level building blocks to build the
+models yourself.
+
+Example implementations for all models can be found here:
+`Model Examples <https://docs.lightly.ai/examples/models.html>`_
 
 The package contains an implementation of the commonly used ResNet and
 adaptations of the architecture which make self-supervised learning simpler.
